"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ExtendedSchema = void 0;
const ValidationResults_1 = require("../models/ValidationResults");
const EMAIL_REGEX = new RegExp(/^(([^<>()[\]\\.,;:\s@"]+(\.[^<>()[\]\\.,;:\s@"]+)*)|(".+"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/);
const valueIsObject = (value) => {
    return typeof value === 'object' && !Array.isArray(value) && value !== null;
};
class ExtendedSchema {
    constructor(rules) {
        this.fields = {};
        this.rules = rules || {
            excess: 'clean',
        };
    }
    addField(key, type) {
        this.fields[key] = type;
    }
    addExtendedSchemaField(key, schema, callback, allowUndefined) {
        this.fields[key] = {
            typesToCheck: ['extSchema'],
            callback,
            allowUndefined,
            schema,
        };
    }
    // eslint-disable-next-line @typescript-eslint/explicit-module-boundary-types
    check(value) {
        var _a, _b;
        try {
            if (!valueIsObject(value)) {
                return (0, ValidationResults_1.WRONG_TYPE_INPUT)();
            }
            const valueKeys = Object.keys(value);
            const schemaKeys = Object.keys(this.fields);
            if (valueKeys.length > schemaKeys.length && ((_a = this.rules) === null || _a === void 0 ? void 0 : _a.excess) === 'forbid') {
                return (0, ValidationResults_1.EXCESS_KEYS_INPUT)();
            }
            const cleanValue = Object.assign({}, value);
            const encounteredKeys = new Set();
            for (const key in this.fields) {
                const field = this.fields[key];
                const valueToCheck = value[key];
                if (valueToCheck === undefined) {
                    if (!field.undefined || field.undefined === 'forbid') {
                        return (0, ValidationResults_1.WRONG_TYPE_INPUT)();
                    }
                    if (field.callback && !field.callback(valueToCheck)) {
                        return (0, ValidationResults_1.WRONG_TYPE_INPUT)();
                    }
                    encounteredKeys.add(key);
                    continue;
                }
                const type = typeof valueToCheck;
                for (const typeToCheck of field.typesToCheck) {
                    if (type === typeToCheck) {
                        continue;
                    }
                    if (typeToCheck === 'array' && Array.isArray(valueToCheck)) {
                        // because typeof [] === 'object'
                        continue;
                    }
                    if (typeToCheck === 'extSchema' && valueIsObject(valueToCheck)) {
                        if (!('schema' in field) || (field === null || field === void 0 ? void 0 : field.schema) === undefined) {
                            return (0, ValidationResults_1.INTERNAL_ERROR)();
                        }
                        const result = field.schema.check(valueToCheck);
                        if (!result.success) {
                            return (0, ValidationResults_1.WRONG_TYPE_INPUT)();
                        }
                        continue;
                    }
                    if ((typeToCheck === 'object' || typeToCheck === 'extSchema') && valueIsObject(valueToCheck)) {
                        continue;
                    }
                    return (0, ValidationResults_1.WRONG_TYPE_INPUT)();
                }
                if (field.callback && !field.callback(valueToCheck)) {
                    return (0, ValidationResults_1.WRONG_TYPE_INPUT)();
                }
                encounteredKeys.add(key);
            }
            if (encounteredKeys.size < schemaKeys.length) {
                console.log(encounteredKeys, schemaKeys);
                return (0, ValidationResults_1.MISSING_KEYS_INPUT)();
            }
            if (((_b = this.rules) === null || _b === void 0 ? void 0 : _b.excess) === 'clean') {
                for (const key in value) {
                    if (!encounteredKeys.has(key)) {
                        delete cleanValue[key];
                    }
                }
            }
            return (0, ValidationResults_1.VALID_INPUT)(cleanValue);
        }
        catch (error) {
            console.log(error);
            return (0, ValidationResults_1.INTERNAL_ERROR)();
        }
    }
    length() {
        return Object.keys(this.fields).length;
    }
    toObject() {
        // we convert the schema to TypeScript like type
        const obj = {};
        for (const [key, field] of Object.entries(this.fields)) {
            if (field.typesToCheck)
                obj[key] = field.typesToCheck.length === 1 ? field.typesToCheck[0] : field.typesToCheck.join(' | ');
        }
        return obj;
    }
    /* SUGAR */
    addStringField(key, options) {
        this.addField(key, {
            typesToCheck: ['string'],
            callback: options === null || options === void 0 ? void 0 : options.callback,
            undefined: options === null || options === void 0 ? void 0 : options.undefined,
        });
    }
    addNumberField(key, options) {
        this.addField(key, {
            typesToCheck: ['number'],
            callback: options === null || options === void 0 ? void 0 : options.callback,
            undefined: options === null || options === void 0 ? void 0 : options.undefined,
        });
    }
    addBooleanField(key, options) {
        this.addField(key, {
            typesToCheck: ['boolean'],
            callback: options === null || options === void 0 ? void 0 : options.callback,
            undefined: options === null || options === void 0 ? void 0 : options.undefined,
        });
    }
    addArrayField(key, options) {
        this.addField(key, {
            typesToCheck: ['array'],
            callback: options === null || options === void 0 ? void 0 : options.callback,
            undefined: options === null || options === void 0 ? void 0 : options.undefined,
        });
    }
    addEmailField(key, options) {
        this.addRegexField(key, EMAIL_REGEX, options);
    }
    addRegexField(key, regex, options) {
        this.addField(key, {
            typesToCheck: ['string'],
            callback: (value) => {
                if (!regex.test(value)) {
                    return false;
                }
                return (options === null || options === void 0 ? void 0 : options.callback) ? options.callback(value) : true;
            },
            undefined: options === null || options === void 0 ? void 0 : options.undefined,
        });
    }
    addFalsyField(key, options) {
        this.addBooleanField(key, {
            callback: (value) => {
                return !value;
            },
            undefined: options === null || options === void 0 ? void 0 : options.undefined,
        });
    }
    addTruthyField(key, options) {
        this.addBooleanField(key, {
            callback: (value) => {
                return !!value;
            },
            undefined: options === null || options === void 0 ? void 0 : options.undefined,
        });
    }
    addArrayOfElementsField(key, elementSchema, options) {
        this.addArrayField(key, {
            callback: (value) => {
                if (!Array.isArray(value)) {
                    return false;
                }
                for (const element of value) {
                    const result = elementSchema.check(element);
                    if (!result.success) {
                        return false;
                    }
                }
                return !(options === null || options === void 0 ? void 0 : options.callback) || options.callback(value);
            },
            undefined: options === null || options === void 0 ? void 0 : options.undefined,
        });
    }
}
exports.ExtendedSchema = ExtendedSchema;
